@@ -3,7 +3,6 @@ package client
 import (
 	"bytes"
 	"context"
-	"encoding/json"
 	"fmt"
 	"net/http"
 	"path"
@@ -13,46 +12,50 @@ import (
 	clairerror "github.com/quay/clair/v4/clair-error"
 	"github.com/quay/clair/v4/httptransport"
 	"github.com/quay/clair/v4/indexer"
+	"github.com/quay/clair/v4/internal/codec"
 )
 
 var _ indexer.Service = (*HTTP)(nil)
 
-func (s *HTTP) AffectedManifests(ctx context.Context, v []claircore.Vulnerability) (claircore.AffectedManifests, error) {
-	var affected claircore.AffectedManifests
-	buf := bytes.NewBuffer([]byte{})
-	err := json.NewEncoder(buf).Encode(struct {
+func (s *HTTP) AffectedManifests(ctx context.Context, v []claircore.Vulnerability) (*claircore.AffectedManifests, error) {
+	u, err := s.addr.Parse(httptransport.AffectedManifestAPIPath)
+	if err != nil {
+		return nil, fmt.Errorf("failed to parse api address: %v", err)
+	}
+	rd := codec.JSONReader(struct {
 		V []claircore.Vulnerability `json:"vulnerabilities"`
 	}{
 		v,
 	})
+	req, err := http.NewRequestWithContext(ctx, http.MethodPost, u.String(), rd)
 	if err != nil {
-		return affected, &clairerror.ErrBadVulnerabilities{err}
-	}
-
-	u, err := s.addr.Parse(httptransport.AffectedManifestAPIPath)
-	if err != nil {
-		return affected, fmt.Errorf("failed to parse api address: %v", err)
-	}
-	req, err := http.NewRequestWithContext(ctx, "POST", u.String(), buf)
-	if err != nil {
-		return affected, fmt.Errorf("failed to create request: %v", err)
+		return nil, fmt.Errorf("failed to create request: %v", err)
 	}
+	req.Header.Set("content-type", `application/json`)
 	resp, err := s.c.Do(req)
-	if resp != nil {
-		defer resp.Body.Close()
-	}
 	if err != nil {
-		return affected, fmt.Errorf("request failed: %v", err)
+		return nil, err
 	}
 	defer resp.Body.Close()
 	if resp.StatusCode != http.StatusOK {
-		return affected, &clairerror.ErrRequestFail{Code: resp.StatusCode, Status: resp.Status}
+		return nil, &clairerror.ErrRequestFail{
+			Code:   resp.StatusCode,
+			Status: resp.Status,
+		}
 	}
-	err = json.NewDecoder(resp.Body).Decode(&affected)
-	if err != nil {
-		return affected, &clairerror.ErrBadAffectedManifests{err}
+
+	var a claircore.AffectedManifests
+	switch ct := req.Header.Get("content-type"); ct {
+	case "", `application/json`:
+		dec := codec.GetDecoder(resp.Body)
+		defer codec.PutDecoder(dec)
+		if err := dec.Decode(&a); err != nil {
+			return nil, err
+		}
+	default:
+		return nil, fmt.Errorf("unrecognized content-type %q", ct)
 	}
-	return affected, nil
+	return &a, nil
 }
 
 // Index receives a Manifest and returns a IndexReport providing the indexed
@@ -62,38 +65,40 @@ func (s *HTTP) AffectedManifests(ctx context.Context, v []claircore.Vulnerabilit
 // could not start. If an error occurs during the index operation the error will
 // be preset on the IndexReport.Err field of the returned IndexReport.
 func (s *HTTP) Index(ctx context.Context, manifest *claircore.Manifest) (*claircore.IndexReport, error) {
-	buf := bytes.NewBuffer([]byte{})
-	err := json.NewEncoder(buf).Encode(manifest)
-	if err != nil {
-		return nil, &clairerror.ErrBadManifest{err}
-	}
-
 	u, err := s.addr.Parse(httptransport.IndexAPIPath)
 	if err != nil {
 		return nil, fmt.Errorf("failed to create request: %v", err)
 	}
-	req, err := http.NewRequestWithContext(ctx, "POST", u.String(), buf)
+
+	req, err := http.NewRequestWithContext(ctx, http.MethodPost, u.String(), codec.JSONReader(manifest))
 	if err != nil {
 		return nil, fmt.Errorf("failed to create request: %v", err)
 	}
+	req.Header.Set("content-type", `application/json`)
 	resp, err := s.c.Do(req)
-	if resp != nil {
-		defer resp.Body.Close()
-	}
 	if err != nil {
 		return nil, fmt.Errorf("failed to do request: %v", err)
 	}
+	defer resp.Body.Close()
 	if resp.StatusCode != http.StatusOK {
-		return nil, &clairerror.ErrRequestFail{Code: resp.StatusCode, Status: resp.Status}
+		return nil, &clairerror.ErrRequestFail{
+			Code:   resp.StatusCode,
+			Status: resp.Status,
+		}
 	}
 
-	var sr *claircore.IndexReport
-	err = json.NewDecoder(resp.Body).Decode(sr)
-	if err != nil {
-		return nil, &clairerror.ErrBadIndexReport{err}
+	var ir claircore.IndexReport
+	switch ct := resp.Header.Get("content-type"); ct {
+	case "", `application/json`:
+		dec := codec.GetDecoder(resp.Body)
+		defer codec.PutDecoder(dec)
+		if err := dec.Decode(&ir); err != nil {
+			return nil, err
+		}
+	default:
+		return nil, fmt.Errorf("unrecognized content-type %q", ct)
 	}
-
-	return sr, nil
+	return &ir, nil
 }
 
 // IndexReport retrieves a IndexReport given a manifest hash string
@@ -102,30 +107,34 @@ func (s *HTTP) IndexReport(ctx context.Context, manifest claircore.Digest) (*cla
 	if err != nil {
 		return nil, false, fmt.Errorf("failed to create request: %v", err)
 	}
-	req, err := http.NewRequestWithContext(ctx, "GET", u.String(), nil)
+
+	req, err := http.NewRequestWithContext(ctx, http.MethodGet, u.String(), nil)
 	if err != nil {
 		return nil, false, fmt.Errorf("failed to create request: %v", err)
 	}
 	resp, err := s.c.Do(req)
-	if resp != nil {
-		defer resp.Body.Close()
-	}
 	if err != nil {
 		return nil, false, fmt.Errorf("failed to do request: %v", err)
 	}
-	if resp.StatusCode == http.StatusNotFound {
+	defer resp.Body.Close()
+	switch resp.StatusCode {
+	case http.StatusOK:
+	case http.StatusNotFound:
 		return nil, false, nil
-	}
-	if resp.StatusCode != http.StatusOK {
-		return nil, false, &clairerror.ErrIndexReportRetrieval{&clairerror.ErrRequestFail{Code: resp.StatusCode, Status: resp.Status}}
+	default:
+		return nil, false, &clairerror.ErrIndexReportRetrieval{
+			E: &clairerror.ErrRequestFail{
+				Code:   resp.StatusCode,
+				Status: resp.Status,
+			}}
 	}
 
 	ir := &claircore.IndexReport{}
-	err = json.NewDecoder(resp.Body).Decode(ir)
-	if err != nil {
-		return nil, false, &clairerror.ErrBadIndexReport{err}
+	dec := codec.GetDecoder(resp.Body)
+	defer codec.PutDecoder(dec)
+	if err := dec.Decode(ir); err != nil {
+		return nil, false, &clairerror.ErrBadIndexReport{E: err}
 	}
-
 	return ir, true, nil
 }
 
@@ -134,18 +143,15 @@ func (s *HTTP) State(ctx context.Context) (string, error) {
 	if err != nil {
 		return "", fmt.Errorf("failed to create request: %v", err)
 	}
-	req, err := http.NewRequestWithContext(ctx, "GET", u.String(), nil)
+	req, err := http.NewRequestWithContext(ctx, http.MethodGet, u.String(), nil)
 	if err != nil {
 		return "", fmt.Errorf("failed to create request: %v", err)
 	}
 	resp, err := s.c.Do(req)
-	if resp != nil {
-		defer resp.Body.Close()
-	}
-
 	if err != nil {
 		return "", fmt.Errorf("failed to do request: %v", err)
 	}
+	defer resp.Body.Close()
 	buf := &bytes.Buffer{}
 	if _, err := buf.ReadFrom(resp.Body); err != nil {
 		return "", err
